#ifndef HARKIO_HARK_HARKFILE_HPP_
#define HARKIO_HARK_HARKFILE_HPP_

#include <string>
#include <vector>
#include <stdexcept>
#include <boost/shared_ptr.hpp>
#include <harkio2/hark/Writable.hpp>
#include <harkio2/hark/HarkFileWriter.hpp>

/** @file HarkFile.hpp
 * @brief HarkFileクラスの定義
 */

namespace harkio2
{
namespace hark
{

class DataSet;

/** @brief HARKファイルを表す。
 *
 * このクラスは、HARKファイルの構造を特定のファイル形式（バイナリ、XMLなど）から切り離すために、
 * 抽象的なHARKファイルの構造を表す。
 *
 * あるHARKファイルはいくつかのデータセットから構成される。
 * （バイナリ形式のHARKファイルの場合、一つのデータセットは一つのチャンクに相当する。）
 * \sa DataSet
 */
class HarkFile : public Writable
{
public:
    /** @brief 指定されたインデックスにあるデータセットを返す。
     * @param index データセットのインデックス
     * @return 指定されたインデックスにあるデータセット
     * @throw std::out_of_range 与えられたインデックスが範囲外である。
     */
    boost::shared_ptr<harkio2::hark::DataSet> GetDataSet(size_t index) const throw (std::out_of_range);

    /** @brief 指定されたIDを持つデータセットのうち、最初に見つかったものを返す。見つからないならnullを返す。
     *
     * 次の条件を満たす最小のインデックスiを持つデータセットを返す。
     * (id == GetDataSet(i)->GetId())
     * \sa DataSet::GetId
     * @param id 検索するID
     * @return 指定されたIDを持つデータセット。見つからないならnull
     */
    boost::shared_ptr<harkio2::hark::DataSet> GetDataSet(const std::string& id) const throw ();

    /** @brief 指定されたIDを持つデータセットをすべて返す。1つも見つからないなら空配列を返す。
     * @param id 検索するID
     * @return 見つかったデータセットの配列
     */
    std::vector<boost::shared_ptr<harkio2::hark::DataSet> > SearchDataSets(const std::string& id) const throw ();

    /** @brief 指定されたデータセットをリストの末尾に追加する。
     * @param dataSet 追加するデータセット
     */
    void AddDataSet(boost::shared_ptr<harkio2::hark::DataSet> dataSet) throw ();

    /** @brief 指定されたインデックスのデータセットを削除する。
     * @param index 削除するデータセットのインデックス
     * @throw std::out_of_range 指定されたインデックスが範囲外である。
     */
    void RemoveDataSet(size_t index) throw (std::out_of_range);

    /** @brief 指定されたデータセットを削除する。
     *
     * 次の条件を満たす最小のインデックスiを持つデータセットを削除する。
     * (dataSet.get() == GetDataSet(i).get())<BR>
     * この条件式は、指定されたデータセットとポインタ値が等しいかどうかを検査している。
     * つまり、内容が等しくても、異なるインスタンスのデータセットは削除されない。
     * @param dataSet このHarkFileから削除するデータセット
     */
    void RemoveDataSet(boost::shared_ptr<harkio2::hark::DataSet> dataSet) throw ();

    /** @brief 指定されたIDを持つデータセットのうち、最初に見つかったものを削除する。
     *
     * 次の条件を満たす最小のインデックスiを持つデータセットを削除する。
     * (id == GetDataSet(i)->GetId())
     * \sa DataSet::GetId
     * @param id 検索するID
     */
    void RemoveDataSet(const std::string& id) throw ();

    /** @brief コンテナに格納されているデータセット数を返す。
     *
     * GetDataSet や RemoveDataSet メソッドにインデックスを指定する場合、
     * そのインデックスはデータセット数より小さくなければならない。
     * @return コンテナに格納されているデータセット数
     */
    size_t GetDataSetsSize() const throw ();

    /** @brief データ構造を指定されたライターを使って書き出す。
     *
     * 内部的に利用されるメソッドのため、一般ユーザーは使う必要はありません。
     * @param writer ライターオブジェクト
     */
    void Write(HarkFileWriter& writer) throw (harkio2::InvalidHarkFile)
    {
        writer.Write(*this);
    }

private:
    std::vector<boost::shared_ptr<harkio2::hark::DataSet> > dataSets;
};

/** @brief 指定されたHarkFileが有効な状態かどうかを返す。
 *
 * 「有効」とは、HarkFileが特定のDataSetを持っていることを意味する。
 * HarkFileが余計なデータセットを含んでいたり、
 * 必要なデータセットを含んでいない場合は無効な状態である。
 *
 * どのようなデータセットの組が有効なのかは、hark-documentを参照してください。
 */
bool IsValidHarkFile(const harkio2::hark::HarkFile& harkFile) throw ();

}
}

#endif // HARKIO_HARK_HARKFILE_HPP_

