#ifndef HARKIO_HARK_HARKFILEWRITER_HPP_
#define HARKIO_HARK_HARKFILEWRITER_HPP_

#include <ostream>
#include <boost/shared_ptr.hpp>
#include <harkio2/InvalidHarkFile.hpp>

/** @file HarkFileWriter.hpp
 * @brief HarkFileWriter クラスの定義
 */

namespace harkio2
{
namespace hark
{

class HarkFile;
namespace hgtf
{
    class Fmt1_0;
    class Spec1_0;
    class M2PG1_0;
    class GTF1_0;
    class SM1_0;
}

/** @brief HARKファイルの書き込みインターフェース。
 *
 * このインターフェースを実装した、各種ファイル形式（例えばバイナリ形式）
 * に対応したクラスを用いることにより、様々なフォーマットのHARKファイルへデータを書き出すことができる。
 *
 * @sa HarkFileBinaryWriter
 */
class HarkFileWriter
{
public:
    /// @brief 継承のための仮想デストラクタ
    virtual ~HarkFileWriter() {}

    /** @brief HARKファイルへ指定されたデータを書き出す。
     *
     * HARKファイルのデータをどこかへ書き出す。
     * 書き出す場所はサブクラスのコンストラクタやメソッドにより指定されるだろう。
     *
     * このインターフェースでは、HARKファイルが本当のファイルであるかどうかは規定していないため、
     * 例えば名前付きパイプなどへ書き出すようなサブクラスを定義することもできる。
     * @param hark 書き出すデータ
     */
    virtual void Write(boost::shared_ptr<harkio2::hark::HarkFile> hark) throw (harkio2::InvalidHarkFile) = 0;

    /*
     * 各種データ構造用の書き出しメソッド
     * データ構造を新設するたびに追加する必要がある
     */
    virtual void Write(const harkio2::hark::HarkFile& o) throw (harkio2::InvalidHarkFile) = 0;
    virtual void Write(const harkio2::hark::hgtf::Fmt1_0& o) throw (harkio2::InvalidHarkFile) = 0;
    virtual void Write(const harkio2::hark::hgtf::Spec1_0& o) throw (harkio2::InvalidHarkFile) = 0;
    virtual void Write(const harkio2::hark::hgtf::M2PG1_0& o) throw (harkio2::InvalidHarkFile) = 0;
    virtual void Write(const harkio2::hark::hgtf::GTF1_0& o) throw (harkio2::InvalidHarkFile) = 0;
    virtual void Write(const harkio2::hark::hgtf::SM1_0& o) throw (harkio2::InvalidHarkFile) = 0;
};

}
}

#endif // HARKIO_HARK_HARKFILEWRITER_HPP_

